#include "STC15Wxx.h"
#include "intrins.h"
#include "tm1640.h"

#define uint unsigned int
#define uchar unsigned char
#define ulong unsigned long

#define ADC_POWER   0x80            //ADC电源控制位
#define ADC_FLAG    0x10            //ADC完成标志
#define ADC_START   0x08            //ADC起始控制位
#define ADC_SPEEDLL 0x00            //540个时钟

#define VOLTAGE_BASE_CH 0
#define VOLTAGE_3V3_CH 4
#define VOLTAGE_5V_CH 2
#define VOLTAGE_5VSB_CH 1
#define VOLTAGE_12V_CH 3
#define VOLTAGE_N12V_CH 5
#define BASE_VOLTAGE 2.515f

#define BUF_SIZE 24

sbit T = P2^5;


void Delay1000ms();
void Delay10ms();
void initADC();
void readBaseVoltage();
void readVoltage3v3();
void readVoltage5v();
void readVoltage5vSB();
void readVoltage12v();
void readVoltageN12v();
uint getADCResult(uchar ch);

float voltagePerBit;

uint baseVoltageBuffer[BUF_SIZE + 1], voltage3v3Buffer[BUF_SIZE + 1], voltage5vBuffer[BUF_SIZE + 1], voltage5vSBBuffer[BUF_SIZE + 1], voltage12vBuffer[BUF_SIZE + 1], voltageN12vBuffer[BUF_SIZE + 1];
uchar samplingCountBase=0, samplingCount3V3, samplingCount5V, samplingCount5VSB, samplingCount12V,samplingCountN12V;
uint voltage3v3Mv=0, voltage5vMv=0, voltage5vSBMv=0, voltage12vMv=0, voltageN12vMv=0; 

void main()
{
	uint count;

	P1M1 = 0xFF;
	P1M0 = 0x00;
	
	initADC();
    TM1640Init(2, 0xff);
	
	while(1){
        T = ~T;
		if(samplingCountBase < BUF_SIZE){
			readBaseVoltage();
			Delay10ms();
		}
		readVoltage3v3();
		Delay10ms();
		readVoltage5v();
		Delay10ms();
		readVoltage5vSB();
		Delay10ms();
		readVoltageN12v();
		Delay10ms();
		readVoltage12v();
		Delay10ms();
		
		// 读取10次电压刷新一次显示
		if(count++ > 10){
			count = 0;
			// 3v3
			TM1640SetNumber(9, voltage3v3Mv / 1000, 1);
			TM1640SetNumber(10, voltage3v3Mv % 1000 / 100, 0);
			TM1640SetNumber(11, voltage3v3Mv % 100 / 10, 0);
			
			// 5v
			TM1640SetNumber(0, voltage5vMv / 1000, 1);
			TM1640SetNumber(1, voltage5vMv % 1000 / 100, 0);
			TM1640SetNumber(2, voltage5vMv % 100 / 10, 0);
			
			// 5vSB
			TM1640SetNumber(6, voltage5vSBMv / 1000, 1);
			TM1640SetNumber(7, voltage5vSBMv % 1000 / 100, 0);
			TM1640SetNumber(8, voltage5vSBMv % 100 / 10, 0);
			
			// -12V
			TM1640SetNumber(3, voltageN12vMv / 10000, 0);
			TM1640SetNumber(4, voltageN12vMv % 10000 / 1000, 1);
			TM1640SetNumber(5, voltageN12vMv % 1000 / 100, 0);
			
			// 12V
			TM1640SetNumber(12, voltage12vMv / 10000, 0);
			TM1640SetNumber(13, voltage12vMv % 10000 / 1000, 1);
			TM1640SetNumber(14, voltage12vMv % 1000 / 100, 0);
			TM1640SetNumber(15, voltage12vMv % 100 / 10, 0);

		}
	}	
}
 
void readBaseVoltage(){
	uint value,i,sum = 0;
	value = getADCResult(VOLTAGE_BASE_CH);
	baseVoltageBuffer[BUF_SIZE] = value;
	if(++samplingCountBase < BUF_SIZE){
		//准备递推滤波算法的数据
        for(i = 0; i < BUF_SIZE; i++)  
		{
			//所有数据循环左移
			baseVoltageBuffer[i] = baseVoltageBuffer[i+1];
		}
	}
	else{
		samplingCountBase = BUF_SIZE;
		//递推滤波算法
		 for(i = 0; i < BUF_SIZE; i++) 
		{
			//所有数据循环左移
			baseVoltageBuffer[i] = baseVoltageBuffer[i+1];
			//求和
			sum += baseVoltageBuffer[i];  
		}
		//求平均值
		value = sum / BUF_SIZE;        
	}
	voltagePerBit = BASE_VOLTAGE / value;
}

void readVoltage3v3(){
	uint value,i,sum = 0;
	value = getADCResult(VOLTAGE_3V3_CH);
	voltage3v3Buffer[BUF_SIZE] = value;
	if(++samplingCount3V3 < BUF_SIZE){
		//准备递推滤波算法的数据
        for(i = 0; i < BUF_SIZE; i++)  
		{
			//所有数据循环左移
			voltage3v3Buffer[i] = voltage3v3Buffer[i+1];
		}
	}
	else{
		samplingCount3V3 = BUF_SIZE;
		//递推滤波算法
		 for(i = 0; i < BUF_SIZE; i++) 
		{
			//所有数据循环左移
			voltage3v3Buffer[i] = voltage3v3Buffer[i+1];
			//求和
			sum += voltage3v3Buffer[i];  
		}
		//求平均值
		value = sum / BUF_SIZE;        
	}
	
	voltage3v3Mv = value * voltagePerBit * 1000;
}

void readVoltage5v(){
	uint value,i,sum = 0;
	value = getADCResult(VOLTAGE_5V_CH);
	voltage5vBuffer[BUF_SIZE] = value;
	if(++samplingCount5V < BUF_SIZE){
		//准备递推滤波算法的数据
        for(i = 0; i < BUF_SIZE; i++)  
		{
			//所有数据循环左移
			voltage5vBuffer[i] = voltage5vBuffer[i+1];
		}
	}
	else{
		samplingCount5V = BUF_SIZE;
		//递推滤波算法
		 for(i = 0; i < BUF_SIZE; i++) 
		{
			//所有数据循环左移
			voltage5vBuffer[i] = voltage5vBuffer[i+1];
			//求和
			sum += voltage5vBuffer[i];  
		}
		//求平均值
		value = sum / BUF_SIZE;        
	}
	
	voltage5vMv = value * voltagePerBit * 1500;
}

void readVoltage5vSB(){
	uint value,i,sum = 0;
	value = getADCResult(VOLTAGE_5VSB_CH);
	voltage5vSBBuffer[BUF_SIZE] = value;
	if(++samplingCount5VSB < BUF_SIZE){
		//准备递推滤波算法的数据
        for(i = 0; i < BUF_SIZE; i++)  
		{
			//所有数据循环左移
			voltage5vSBBuffer[i] = voltage5vSBBuffer[i+1];
		}
	}
	else{
		samplingCount5VSB = BUF_SIZE;
		//递推滤波算法
		 for(i = 0; i < BUF_SIZE; i++) 
		{
			//所有数据循环左移
			voltage5vSBBuffer[i] = voltage5vSBBuffer[i+1];
			//求和
			sum += voltage5vSBBuffer[i];  
		}
		//求平均值
		value = sum / BUF_SIZE;        
	}
	
	voltage5vSBMv = value * voltagePerBit * 1500;
}

void readVoltage12v(){
	uint value,i,sum = 0;
	value = getADCResult(VOLTAGE_12V_CH);
	voltage12vBuffer[BUF_SIZE] = value;
	if(++samplingCount12V < BUF_SIZE){
		//准备递推滤波算法的数据
        for(i = 0; i < BUF_SIZE; i++)  
		{
			//所有数据循环左移
			voltage12vBuffer[i] = voltage12vBuffer[i+1];
		}
	}
	else{
		samplingCount12V = BUF_SIZE;
		//递推滤波算法
		 for(i = 0; i < BUF_SIZE; i++) 
		{
			//所有数据循环左移
			voltage12vBuffer[i] = voltage12vBuffer[i+1];
			//求和
			sum += voltage12vBuffer[i];  
		}
		//求平均值
		value = sum / BUF_SIZE;        
	}
	
	voltage12vMv = value * voltagePerBit * 3039;
}

void readVoltageN12v(){
	uint value,i,sum = 0;
	value = getADCResult(VOLTAGE_N12V_CH);
	voltageN12vBuffer[BUF_SIZE] = value;
	if(++samplingCountN12V < BUF_SIZE){
		//准备递推滤波算法的数据
        for(i = 0; i < BUF_SIZE; i++)  
		{
			//所有数据循环左移
			voltageN12vBuffer[i] = voltageN12vBuffer[i+1];
		}
	}
	else{
		samplingCountN12V = BUF_SIZE;
		//递推滤波算法
		 for(i = 0; i < BUF_SIZE; i++) 
		{
			//所有数据循环左移
			voltageN12vBuffer[i] = voltageN12vBuffer[i+1];
			//求和
			sum += voltageN12vBuffer[i];  
		}
		//求平均值
		value = sum / BUF_SIZE;        
	}
	
	voltageN12vMv = value * voltagePerBit * 3000;
}

void initADC(){
	P1ASF = 0xff;                   //设置P1口为AD口
    ADC_RES = 0;                    //清除结果寄存器
    ADC_CONTR = ADC_POWER | ADC_SPEEDLL;
	CLK_DIV |= 0x20;
    _nop_(); _nop_();_nop_();_nop_();  //ADC上电并延时
}

uint getADCResult(uchar ch)
{
	uint result;
	
    ADC_CONTR = ADC_POWER | ADC_SPEEDLL | ch | ADC_START;
    _nop_();                        //等待4个NOP
    _nop_();
    _nop_();
    _nop_();
    while (!(ADC_CONTR & ADC_FLAG));//等待ADC转换完成
    ADC_CONTR &= ~ADC_FLAG;         //Close ADC

	result = (ADC_RES << 8) | ADC_RESL;
	
    return result;                 //返回ADC结果
}

void Delay1000ms()		//@11.0592MHz
{
	unsigned char i, j, k;

	_nop_();
	_nop_();
	i = 43;
	j = 6;
	k = 203;
	do
	{
		do
		{
			while (--k);
		} while (--j);
	} while (--i);
}

void Delay10ms()		//@11.0592MHz
{
	unsigned char i, j;

	i = 108;
	j = 145;
	do
	{
		while (--j);
	} while (--i);
} 
