#include "tm1640.h"
#define TM1640_DELAY {_nop_();_nop_();}
#define TM1640_DELAY_HALF _nop_();

void shiftOut(unsigned char dat);
void start();
void stop();
uchar translateFromNumber(uchar number);
uchar translateFromChar(char number);

void TM1640Init(uchar brightness, uchar initChar){
	uchar i;
	brightness += 7;
	
	start();
	shiftOut(0x40);
	stop();
	
	start();
    shiftOut(0xC0); 
    for(i = 0; i < 16; i++){
        shiftOut(initChar);
    }
	stop();
	
	start();
	shiftOut(0x80 | (brightness & 0x0F));
    
	stop();
}

void TM1640SetBrightness(uchar brightness){
	brightness += 7;
	start();
	shiftOut(0x80 | (brightness & 0x0F));
	stop();
}

void TM1640SetNumber(uchar index, uchar number, uchar point){
	uchar raw;
	raw = translateFromNumber(number);
	if(point){
		raw = raw|0x80;
	}
	TM1640SetRawDigit(index, raw);
}

void TM1640SetChar(uchar index, char c){
	TM1640SetRawDigit(index, translateFromChar(c));
}

void TM1640SetRawDigit(uchar index, uchar digit){
	start();
	shiftOut(0x44); // 固定地址模式
	stop();
	
	start();
	shiftOut(0xC0 | (index&0x0F)); // 固定地址模式
	shiftOut(digit);
	stop();
}

void TM1640SetRawDigits(uchar startIndex, uchar length, uchar *digits){
	uchar count;
	
	count = 16 - (0x0F & startIndex);
	count = count > length ? length : count;
	
	start();
	shiftOut(0x40); // 连续地址模式
	stop();
	
	start();
	shiftOut(0xC0 | (0x0F & startIndex));
	do{
		shiftOut(*digits++);
	}while(--count != 0);
	stop();
}

void start()
{
    TM1640_DIN=1;
    TM1640_DELAY 
    TM1640_SCLK=1;    
    TM1640_DELAY
    TM1640_DIN=0;
    TM1640_DELAY
    TM1640_SCLK=0;
    TM1640_DELAY
}

void stop()
{
    TM1640_SCLK=0;
    TM1640_DELAY
    TM1640_DIN=0;
    TM1640_SCLK=1;
    TM1640_DELAY 
    TM1640_DIN=1;
    TM1640_DELAY
    TM1640_SCLK=1;
    TM1640_DELAY    
}


void shiftOut(uchar date){
    uchar i,aa;
    aa=date;
    //sda=0;
    //scl=0;
    for(i=0;i<8;i++)
    {     
        TM1640_SCLK=0; 
        if(aa&0x01)        
          {
            TM1640_DIN=1;
            _nop_();
            _nop_(); 
          }
        else
          {
            TM1640_DIN=0;
                _nop_();
                _nop_();  
            }
        TM1640_SCLK=1;
        _nop_();
        aa=aa>>1;
    }
    //scl=0;
    //sda=0;   
}

uchar translateFromNumber(uchar number){
	switch(number){
		case 0: return 0x3F;
		case 1: return 0x06;
		case 2: return 0x5B;
		case 3: return 0x4F;
		case 4: return 0x66;
		case 5: return 0x6D;
		case 6: return 0x7D;
		case 7: return 0x07;
		case 8: return 0x7F;
		case 9: return 0x6F;
		case 10: return 0x77;
		case 11: return 0x7C;
		case 12: return 0x39;
		case 13: return 0x5E;
		case 14: return 0x79;
		case 15: return 0x71;
		case 16: return 0x40;
		default: return 0x00;
	}
}

uchar translateFromChar(char c){
	switch(c){
		case '0': return 0x3F;
		case '1': return 0x06;
		case '2': return 0x5B;
		case '3': return 0x4F;
		case '4': return 0x66;
		case '5': return 0x6D;
		case '6': return 0x7D;
		case '7': return 0x07;
		case '8': return 0x7F;
		case '9': return 0x6F;
		case 'a': case 'A':return 0x77;
		case 'b': case 'B': return 0x7C;
		case 'c': return 0x58;
		case 'C': return 0x39;
		case 'd': case 'D': return 0x5E;
		case 'e': case 'E':return 0x79;
		case 'f': case 'F': return 0x71;
		case 'i': return 0x10;
		case 'I': return 0x30;
		case 'l': case 'L': return 0x38;
		case 'o': case 'O': return 0x5C;
		case 'p': case 'P': return 0x73;
		case 'r': case 'R': return 0x50;
		case 's': case 'S': return 0x6D;
		case 'u': return 0x1C;
		case 'U': return 0x3E;
		case '-': return 0x40;
		case '_': return 0x08;
		default: return 0x00;
	}
}
